#version 440

layout(location = 0) in vec4 qt_Vertex;
layout(location = 1) in vec2 qt_MultiTexCoord0;

layout(location = 0) out vec2 coord;

layout(std140, binding = 0) uniform buf {
    mat4 qt_Matrix;
    float qt_Opacity;
    float shift;
    float maximumValue;
    vec4 color;
    float threshold;
    vec4 aboveThresholdColor;
};

void main() {
    coord = qt_MultiTexCoord0 + vec2(shift, 0.0);
    gl_Position = qt_Matrix * qt_Vertex;
}
