// Copyright 2015 Canonical Ltd.
// Automatically generated by the createshapetextures tool.
#ifndef UCLOMIRISHAPETEXTURES_H
#define UCLOMIRISHAPETEXTURES_H

const int shapeTextureCount = 2;
const int shapeTextureWidth = 32;
const int shapeTextureHeight = 32;
const float shapeTextureOffset = 0.0625;
const int shapeTextureDistanceAA = 4;

extern const unsigned char shapeTextureData[2][4097];

const int shapeTextureMipmapWidth = 256;
const int shapeTextureMipmapHeight = 256;
const int shapeTextureMipmapCount = 9;

extern const int shapeTextureMipmapOffset[9];
extern const unsigned char shapeTextureMipmapData[2][349525];

#endif // UCLOMIRISHAPETEXTURES_H
