/*
    Copyright 2015 Canonical Ltd.
*/

 @import url(basic.css);

body {
    font-family:Lomiri, "DejaVu Sans", "Trebuchet MS", sans-serif;
    font-size:100%;
    background-color:#2C001E;
    margin:0;
    padding:0;
    }

div.documentwrapper {
    float:left;
    width:100%;
}

div.bodywrapper {
    margin:0 0 0 0;
}

hr {
    border:1px solid #B1B4B6;
}

div.body {
    background-color:#FFF;
    color:#3E4349;
    font-size:.9em;
    padding:0 30px 30px;
}

div.footer {
    color:#CCC;
    width:100%;
    text-align:center;
    font-size:75%;
    padding:13px 0;
}

div.footer a {
    color:#444;
    text-decoration:underline;
}

div.related {
    background-color:#DD4814;
    line-height:32px;
    color:#fff;
    font-size:.9em;
}

div.related a {
    color:#EEE;
}

div.sphinxsidebar {
    font-size:.75em;
    line-height:1.5em;
}

div.sphinxsidebarwrapper {
    padding:20px 0;
}

div.sphinxsidebar h3,
div.sphinxsidebar h4 {
    font-family:Lomiri, "DejaVu Sans", "Trebuchet MS", sans-serif;
    color:#222;
    font-size:1.2em;
    font-weight:400;
    background-color:#ddd;
    margin:0;
    padding:5px 10px;
}

div.sphinxsidebar p {
    color:#444;
    padding:2px 10px;
}

div.sphinxsidebar ul {
    color:#000;
    margin:10px 5px;
    padding:0;
}

div.sphinxsidebar input {
    border:1px solid #ccc;
    font-family:Lomiri, "DejaVu Sans", "Trebuchet MS", sans-serif;
    font-size:1em;
}

div.sphinxsidebar input[type=text] {
    margin-left:5px;
}

a {
    color:#5E2750;
    text-decoration:none;
}

a:hover {
    color:#DD4814;
    text-decoration:underline;
}

div.body h1 {
    font-family:Lomiri, "DejaVu Sans", "Trebuchet MS", sans-serif;
    font-weight:bold;
    font-size:200%;
    color:#444;
    margin:20px 0 10px;
    padding:5px 0 5px 10px;
}

div.body h2 {
    font-family:Lomiri, "DejaVu Sans", "Trebuchet MS", sans-serif;
    font-size:130%;
    background-color:#CCC;
    margin:20px 0 10px;
    padding:5px 0 5px 10px;
}

div.body h6 {
    font-family:Lomiri, "DejaVu Sans", "Trebuchet MS", sans-serif;
    background-color:#BED4EB;
    font-weight:400;
    color:#333;
    margin:30px 0 10px;
    padding:5px 0 5px 10px;
}

a.headerlink {
    text-decoration:none;
    padding:0 4px;
}

a.headerlink:hover {
    background-color:#AEA79F;
    color:#FFF;
}

div.body p,div.body dd,div.body li {
    line-height:1.5em;
}

div.highlight {
    background-color:#FFF;
}

div.note {
    background-color:#eee;
    border:1px solid #ccc;
}

div.seealso {
    background-color:#ffc;
    border:1px solid #ff6;
}

div.warning {
    background-color:#ffe4e4;
    border:1px solid #f66;
}

p.admonition-title:after {
    content:":";
}

pre {
    background-color:#EFEFEF;
    color:#000;
    line-height:1.1em;
    border:1px solid #C6C9CB;
    font-size:1.2em;
    font-family:"Ubuntu Mono", Monaco, Consolas, "DejaVu Sans Mono", "Lucida Console", monospace;
    margin:1.5em 0;
    padding:6px;
}

tt {
    background-color:#EFEFEF;
    color:#222;
    font-size:1.1em;
    font-family:"Ubuntu Mono", Monaco, Consolas, "DejaVu Sans Mono", "Lucida Console", monospace;
}

.viewcode-back {
    font-family:Lomiri, "DejaVu Sans", "Trebuchet MS", sans-serif;
}

div.viewcode-block:target {
    background-color:#f4debf;
    border-top:1px solid #ac9;
    border-bottom:1px solid #ac9;
}

div.document,div.topic {
 background-color:#eee;
}

div.sphinxsidebar h3 a,div.sphinxsidebar a {
 color:#444;
}

div.admonition p.admonition-title + p,p.admonition-title {
    display:inline;
}
